#!/usr/bin/env python
#############################################################################
#
#  Linux Desktop Testing Project http://ldtp.freedesktop.org
# 
#  Author:
#     Veerapuram Varadhan <v.varadhan@gmail.com>
#     Prashanth Mohan <prashmohan@gmail.com>
#     Venkateswaran S <wenkat.s@gmail.com>
#     A. Nagappan <nagappan@gmail.com>
# 
#  Copyright 2004 - 2006 Novell, Inc.
# 
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Library General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
# 
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
# 
#  You should have received a copy of the GNU Library General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#
#############################################################################

import socket, os, types, sys, string, struct, traceback, time
import threading, re, atexit, random, thread, select
from xml.parsers.expat import ExpatError
from xml.dom.minidom import *
from xml.sax import saxutils

class state:
	ACTIVE = ' active '
	ARMED = ' armed '
	BUSY = ' busy '
	CHECKED = ' checked '
	COLLAPSED = ' collapsed '
	DEFUNCT = ' defunct '
	EDITABLE = ' editable '
	ENABLED = ' enabled '
	EXPANDABLE = ' expandable '
	EXPANDED = ' expanded '
	FOCUSABLE = ' focusable '
	FOCUSED = ' focused '
	HORIZONTAL = ' horizontal '
	ICONIFIED = ' confined '
	MODAL = ' modal '
	MULTILINE = ' multiline '
	MULTISELECTABLE = ' multiselectable '
	OPAQUE = ' opaque '
	PRESSED = ' pressed '
	RESIZABLE = ' resizable '
	SELECTABLE = ' selectable '
	SELECTED = ' selected '
	SENSITIVE = ' sensitive '
	SHOWING = ' showing '
	SINGLELINE = ' singleline '
	STALE = ' stale '
	TRANSIENT = ' transient '
	VERTICAL = ' vertical '
	VISIBLE = ' visible '
	MANAGESDESCENDANTS = ' mangesdescendants '
	INDETERMINATE = ' indenterminate '
	TRUNCATED = ' truncated '
	REQUIRED = ' required '
	INVALIDENTRY = ' invalidentry '
	SUPPORTSAUTOCOMPLETION = ' supportsautocompletion '
	SELECTABLETEXT = ' selectabletext '
	ISDEFAULT = ' isdefault '
	VISITED = ' visited '

	def __or__ (state1, state2):
		return state1 + state2

class error (Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr (self.value)

class LdtpExecutionError (Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr (self.value)

class ConnectionLost (Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr (self.value)

def generatexml (actionName, *args):
	"""This Function will parse some information into an LDTP packet
	INPUT: name --> function_name
	args --> [,window_name[,object_name[,args...]]]

	OUTPUT: returns a string in the LDTP packet (XML lib2) format"""
	# Argument length
	argsLen = len (args)
	
	_xml = '<?xml version=\"1.0\"?>' ## should this header be there in the packets?
	_xml += '<REQUEST>'
	# Fill action name
	_xml = _xml + '<ACTION>' + actionName + '</ACTION>'
	ldtpDebug = os.getenv ('LDTP_DEBUG')
	if ldtpDebug != None:
		print actionName + ' (',
	if argsLen >= 1:
		_xml = _xml + '<ID>' + saxutils.escape(args[0]) + '</ID>'
	if actionName != 'initappmap' and argsLen >= 2:
		_xml = _xml + '<CONTEXT>' + saxutils.escape(args[1]) + '</CONTEXT>'
		if ldtpDebug != None:
			print args[1],
	if argsLen >= 3:
		_xml = _xml + '<COMPONENT>' + saxutils.escape(args[2]) + '</COMPONENT>'
		if ldtpDebug != None:
			print ', ' + args[2],
	if argsLen >= 4:
		_xml = _xml + '<ARGUMENTS>'
		# Fixme: Dirty hack :( start
		if type (args[3]) == list:
			args = args[3]
			args.insert (0, '')
			args.insert (1, '')
			args.insert (2, '')
			argsLen = len (args)
		# Fixme: Dirty hack :( end
		for index in range (3, argsLen):
			if args[index] != None and args[index] != '':
				_xml = _xml + '<ARGUMENT><![CDATA[' + args[index] + ']]></ARGUMENT>'
				if ldtpDebug != None:
					print ', ' + args[index],
		_xml = _xml + '</ARGUMENTS>'
	if actionName == 'initappmap':
		_xml = _xml + '<ARGUMENTS>'
		_xml = _xml + '<ARGUMENT><![CDATA[' + args[1] + ']]></ARGUMENT>'
		_xml = _xml + '</ARGUMENTS>'
		if ldtpDebug != None:
			print '( ' + args[1],
	if ldtpDebug != None:
		print ')'
	_xml += '</REQUEST>'
	return _xml

def getText (nodelist):
	rc = ""
	for node in nodelist:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
	return rc

def getCData (nodelist):
	rc = ""
	for node in nodelist:
		if node.nodeType == node.CDATA_SECTION_NODE:
			rc = rc + node.data
	return rc

def parseobjectlist (xmldata):
	"""Returns the object list"""
	try:
		_objList = None
		dom = parseString (xmldata)
		try:
			_objList   = dom.getElementsByTagName ('OBJECTLIST')
		except IndexError:
			raise LdtpExecutionError ('Invalid Object List')
		if _objList == None:
			raise LdtpExecutionError ('Invalid Object List')
		taglist = []
		for dataelements in _objList:
			for data in dataelements.getElementsByTagName ('OBJECT'):
				taglist.append (getText (data.childNodes))
		return taglist
	except ExpatError, msg:
		raise LdtpExecutionError ('Parsing XML error: ' + str (msg))

def parsexml (xmlpacket):
	"""Returns the value obtained from the server's return LDTP packet"""

	_statusMsg      = None
	_statusCode     = None
	_serverResponse = None
	_responseType   = None
	_requestId      = None
	_serverResponse = None
	_responseObj    = None
	_serverResponseLen = 0

	try:
		dom = parseString (xmlpacket)
		try:
			_responseObj   = dom.getElementsByTagName ('RESPONSE')[0]
			_responseType = 'response'
		except IndexError:
			try:
				_responseObj   = dom.getElementsByTagName ('NOTIFICATION')[0]
				_responseType = 'notification'
			except IndexError, msg:
				raise LdtpExecutionError ('Invalid Response')
		try:
			_responseStatusObj = _responseObj.getElementsByTagName ('STATUS')[0]
			_statusCode = int (getText (_responseStatusObj.getElementsByTagName ('CODE')[0].childNodes))
			_statusMsg  = getText (_responseStatusObj.getElementsByTagName ('MESSAGE')[0].childNodes)
		except ValueError:
			raise LdtpExecutionError ('Invalid Status')
		except IndexError:
			raise LdtpExecutionError ('Invalid Status')
		try:
			data = _responseObj.getElementsByTagName ('DATA')[0]
			_serverResponse    = getCData (data.getElementsByTagName ('VALUE').item (0).childNodes).encode ('utf-8')
			_serverResponseLen = int (getText (data.getElementsByTagName ('LENGTH')[0].childNodes))
		except ValueError:
			raise LdtpExecutionError ('Invalid Data Length')
		except IndexError:
			# Data tag may not be present
			pass
		try:
			_requestId  = getText (_responseObj.getElementsByTagName ('ID')[0].childNodes)
		except IndexError:
			# On notification _requestId will be empty
			pass
	except ExpatError, msg:
		if msg.code == xml.parsers.expat.errors.XML_ERROR_NO_ELEMENTS:
			return None
		if xml.parsers.expat.ErrorString (msg.code) == xml.parsers.expat.errors.XML_ERROR_NO_ELEMENTS:
			return None
		raise LdtpExecutionError ('Parsing XML error: ' + str (msg))

	# Return all the respective values, let the calling function decide what to do with the values
	return _responseType, (_statusCode, _statusMsg, _requestId), (_serverResponseLen, _serverResponse)

# Send given packet to server
def sendpacket (msg):
	flag = False
	try:
		timedelay = os.getenv ('LDTP_DELAY_CMD')
		sendLck.acquire ()
		if timedelay != None:
			try:
				# Delay each command by 'timedelay' seconds
				time.sleep (int (timedelay))
			except IndexError:
				# Default to 5 seconds delay if LDTP_DELAY_CMD
				# env variable is set
				time.sleep (5)
		flag = True
		# Get client socket fd based on thread id
		client = _sockFdPool.get (threading.currentThread ())
		# Pack length (integer value) in network byte order
		msglen = struct.pack ('!i', len (msg))
		# Send message length
		client.send (msglen)
		# Send message
		client.send (msg)
		ldtpDebug = os.getenv ('LDTP_DEBUG')
		if ldtpDebug != None and ldtpDebug == '2':
			print 'Send packet', msg
		sendLck.release ()
	except socket.error, msg:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			sendLck.release ()
		raise LdtpExecutionError ('Server aborted')

def recvpacket (peek_flag = 0, sockfd = None):
	flag = False
	ldtpDebug = os.getenv ('LDTP_DEBUG')
	try:
		recvLck.acquire ()
		flag = True
		client = None
		# Get client socket fd based on thread id
		if sockfd == None:
			client = _sockFdPool.get (threading.currentThread ())
		else:
			client = sockfd
		_responsePacket = None
		client.settimeout (5.0)
		# Hardcoded 4 bytes, as the server sends 4 bytes as packet length
		data = client.recv (4, peek_flag)
		if data == '' or data == None:
			if flag == True:
				# Reason for using the flag:
				# 'Do not call this method when the lock is unlocked.'
				recvLck.release ()
			return None
		_packetSize, = struct.unpack('!i', data)
		if peek_flag == 0 and ldtpDebug != None and ldtpDebug == '2':
			print 'Received packet size', _packetSize
		# MSG_PEEK
                # This flag causes the receive operation to return data from the beginning
		# of the receive queue without removing that data from  the  queue.
		# Thus, a subsequent receive call will return the same data.

		if peek_flag != 0:
			# MSG_PEEK
			_responsePacket = client.recv (4 + _packetSize, peek_flag)
		else:
			_responsePacket = client.recv (_packetSize, peek_flag)
		if peek_flag != 0:
			_pattern = re.compile ('\<\?xml')
			_searchObj = re.search (_pattern, _responsePacket)
			_finalPacket = _responsePacket[_searchObj.start () :]
			_responsePacket = _finalPacket
		recvLck.release ()
		if peek_flag == 0 and ldtpDebug != None and ldtpDebug == '2':
			print 'Received response Packet', _responsePacket
		return _responsePacket
	except struct.error, msg:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		raise LdtpExecutionError ('Invalid packet length ' + str (msg))
	except AttributeError, msg:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		raise LdtpExecutionError ('Error while receiving packet ' + str (msg))
	except socket.timeout:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		if ldtpDebug != None and ldtpDebug == '2':
			print 'Timeout'
		return ''
	except socket.error, msg:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		raise LdtpExecutionError ('Error while receiving packet ' + str (msg))
	except MemoryError, msg:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		raise LdtpExecutionError ('Error while receiving packet ' + str (msg))
	except:
		if flag == True:
			# Reason for using the flag:
			# 'Do not call this method when the lock is unlocked.'
			recvLck.release ()
		raise LdtpExecutionError ('Error while receiving packet')

def peekresponse ():
	return recvpacket (socket.MSG_PEEK)

def getresponse (packetId):
	while True:
		#print 'DEBUG', 'waiting for readflag', threading.currentThread ()
		_readflag.wait ()
		#print 'DEBUG', 'readflag lock released'
		_readflag.clear ()
		ldtpDebug = os.getenv ('LDTP_DEBUG')
		peekResponsePacket = peekresponse ()
		if peekResponsePacket == None:
			if ldtpDebug != None and ldtpDebug == '2':
				print 'Peekresponse None'
			continue
		try:
			_responseType, _responseStatus, _responseData = parsexml (peekResponsePacket)
		except TypeError, msg:
			if ldtpDebug != None and ldtpDebug == '2':
				print 'TypeError', msg
			continue
		# For precautions, we are just checking, whether the packet is notification or not
		if _responseType == 'notification' or _responseType == None:
			continue
		if _responseStatus[2] == packetId:
			if _responseStatus[0] != 0 and ldtpDebug != None:
				print '*** ' + _responseStatus[1]
			packet = recvpacket ()
			# As we have already parsed the packet,
			# we are just returning the parsed packet
			return _responseStatus, _responseData

def invokecallback (clientsocket, _responseType, _responseStatus, _responseData):
	packet = recvpacket (sockfd = clientsocket)
	# As we have already parsed the packet,
	# we are just going to invoke the callback function
	clientsock = None
	try:
		# Create a client socket
		clientsock = socket.socket (socket.AF_UNIX, socket.SOCK_STREAM)
	except socket.error,msg:
		raise LdtpExecutionError ('Error while creating UNIX socket  ' + str (msg))
	try:
		# Connect to server socket
		clientsock.connect (socketpath)
	except TypeError:
		raise ConnectionLost ('Environment LDTP_AUTH_SOCK variable not set')

	global _serverpoll
	# Register newly created socket for polling
	_serverpoll.register (clientsock, select.POLLIN)

	# Add new client socket to socket fd pool
	_sockFdPool[threading.currentThread ()] = clientsock
	callback = callback_functions.get (_responseData[1])
	_notificationflag.set ()
	if (callable (callback)):
		callback ()
	# Unregister newly created socket from polling once its completed
	_serverpoll.unregister (clientsock)

def invokeltfxcallback (clientsocket, callback):
	clientsock = None
	try:
		# Create a client socket
		clientsock = socket.socket (socket.AF_UNIX, socket.SOCK_STREAM)
	except socket.error,msg:
		raise LdtpExecutionError ('Error while creating UNIX socket  ' + str (msg))
	try:
		# Connect to server socket
		clientsock.connect (socketpath)
	except TypeError:
		raise ConnectionLost ('Environment LDTP_AUTH_SOCK variable not set')

	global _serverpoll
	# Register newly created socket for polling
	_serverpoll.register (clientsock, select.POLLIN)

	# Add new client socket to socket fd pool
	_sockFdPool[threading.currentThread ()] = clientsock
	_notificationflag.set ()
	if (callable (callback)):
		callback ()
	# Unregister newly created socket from polling once its completed
	_serverpoll.unregister (clientsock)

class PollServer (threading.Thread):
	def __init__ (self):
		threading.Thread.__init__ (self)
		self._events = None
	def run (self):
		try:
			self.start_polling_server ()
		except:
			try:
				raise LdtpExecutionError (str (traceback.format_exc ()))
			except AttributeError:
				pass
			except:
				pass
	def start_polling_server (self):
		self._serverDisconnected = False
		global _serverpoll
		_serverpoll = select.poll ()
		_serverpoll.register (mainsock, select.POLLIN)

		while True:
			try:
				#print 'DEBUG', 'entering poll'
				self._events = _serverpoll.poll ()
				#print 'DEBUG', 'poll break'
			except socket.error, msg:
				break
			except:
				_serverpoll.unregister (mainsock)
				sys.exit ()
			try:
				if self._events == None:
					break
				for i in range (0, len (self._events)):
					#print 'DEBUG', i
					# if (self._events[i][1] & select.POLLERR or
					#     self._events[i][1] & select.POLLHUP or
					#     self._events[i][1] & select.POLLNVAL):
					# 	print 'DEBUG Disconnected', self._events[i][1]
					# 	print 'DEBUG', self._events[i][1] & select.POLLERR
					# 	print 'DEBUG', self._events[i][1] & select.POLLHUP
					# 	print 'DEBUG', self._events[i][1] & select.POLLNVAL
					# 	# If interrupted stop polling
					# 	self._serverDisconnected = True
					# 	break
					if (self._events[i][1] & select.POLLIN or self._events[i][1] & select.POLLPRI):
						_sockFdPool[threading.currentThread ()] = socket.fromfd (self._events[i][0],
													 socket.AF_UNIX,
													 socket.SOCK_STREAM)
						try:
							# print 'DEBUG', 'handle packet start', \
							# self._events[i][1] & select.POLLIN, \
							# self._events[i][1] & select.POLLPRI
							if self.handle_packet () == None:
								self._serverDisconnected = True
								# _readflag.set ()
								break
						except LdtpExecutionError, msg:
							self._serverDisconnected = True
							# _readflag.set ()
							break
					elif (self._events[i][1] & select.POLLNVAL):
						# Unregister newly created socket from polling once its completed
						_serverpoll.unregister (self._events[i][0])
					else:
						self._serverDisconnected = True
						break
				if self._serverDisconnected == True:
					break
			# Checking this exception due to bug # 333090 comment # 6
			except TypeError:
				_serverpoll.unregister (mainsock)
				sys.exit ()
	def trace_window (self):
		global ltfx_callback_functions
		try:
			if _notificationflag.isSet () == True:
				# Let us allow only one callback function to execute at any point of time
				_notificationflag.wait ()
				_notificationflag.clear ()
				if ltfx_callback_functions == {}:
					_notificationflag.set ()
					return ''
				window_info = commands.getstatusoutput ('digwin -c')
				if window_info[0] != 0:
					_notificationflag.set ()
					return ''
				callback = None
				regexp = re.compile ('\n+')
				window_list = regexp.split (window_info[1])
				for i in ltfx_callback_functions:
					for j in window_list:
						try:
							regexp = re.compile (i)
							if z.search (j).group () != '':
								callback = ltfx_callback_functions.get (i)
								thread.start_new_thread (invokeltfxcallback,
											 (_sockFdPool.get (threading.currentThread ()),
											  callback))
							else:
								_notificationflag.set ()
								return ''
						except AttributeError:
							pass
				_notificationflag.set ()
			else:
				# CPU goes high for some time, if this delay is not there
				# as the notification packet is still in the Queue, but the
				# new spanned thread takes some time to receive the notification packet
				# So this delay is required
				time.sleep (1)
			return ''
		except LdtpExecutionError:
			#_readflag.set ()
			return ''
		except KeyboardInterrupt:
			return None
	def handle_packet (self):
		try:
			self._peekResponsePacket = peekresponse ()
		except KeyboardInterrupt:
			return None
		_responseType = None
		try:
			if self._peekResponsePacket == None:
				# When there is no data to read let us quit
				return None
			if self._peekResponsePacket != '':
				_responseType, _responseStatus, _responseData = parsexml (self._peekResponsePacket)
				if _responseType == None:
					return None
			else:
				_readflag.set ()
		except TypeError:
			return ''
		except LdtpExecutionError:
			#_readflag.set ()
			return ''
		except KeyboardInterrupt:
			return None
		try:
			if _responseType == 'notification':
				if _notificationflag.isSet () == True:
					# Let us allow only one callback function to execute at any point of time
					_notificationflag.wait ()
					_notificationflag.clear ()
					thread.start_new_thread (invokecallback,
								 (_sockFdPool.get (threading.currentThread ()),
								  _responseType, _responseStatus, _responseData))
				else:
					# CPU goes high for some time, if this delay is not there
					# as the notification packet is still in the Queue, but the
					# new spanned thread takes some time to receive the notification packet
					# So this delay is required
					time.sleep (1)
			else:
				# print 'DEBUG readflag set 1'
				_readflag.set ()
		except KeyboardInterrupt:
			return None
		return ''
	def shutdown (self):
		_readflag.set ()

def shutdown ():
	if threading.activeCount () > 1:
		thread.exit ()

# socketpath = os.environ.get ("LDTP_AUTH_SOCK")
socketpath = '/tmp/ldtp-' + os.getenv ('USER') + '-' + os.getenv ('DISPLAY')

# Init callback functions dictionary
callback_functions = {}

# Init callback functions dictionary
ltfx_callback_functions = {}

# Create read flag
_readflag = threading.Event ()
# Clear the flag by default
_readflag.clear ()

# Create notification flag
_notificationflag = threading.Event ()
# Set the flag by default
_notificationflag.set ()

# Contains poll fd's
_serverpoll = None

# Socket fd pool
_sockFdPool = {}

# Send lock
sendLck = threading.Lock ()
# Recieve lock
recvLck = threading.Lock ()

if socketpath == '':
	raise ConnectionLost ('Server not running')
try:
	# Create a client socket
	mainsock = socket.socket (socket.AF_UNIX, socket.SOCK_STREAM)
except socket.error,msg:
	raise LdtpExecutionError ('Error while creating UNIX socket  ' + str (msg))

# Let us retry connecting to the server for 3 times
retryCount = 0

while True:
	try:
		try:
			# Connect to server socket
			mainsock.connect (socketpath)
			break
		except TypeError:
			raise ConnectionLost ('Environment LDTP_AUTH_SOCK variable not set')
	except socket.error, msg:
		if retryCount == 3:
			raise ConnectionLost ('Could not establish connection ' + str (msg))
		retryCount += 1
		_pid = os.fork ()
		if _pid == 0:
			try:
				os.execvpe ('ldtp', (), os.environ)
			except OSError:
				raise LdtpExecutionError ('ldtp executable not in PATH')
		else:
			# Let us wait for 1 second, let the server starts
			time.sleep (1)

_sockFdPool[threading.currentThread ()] = mainsock

# Start polling server
_pollThread = PollServer ()
_pollThread.setDaemon (True)
_pollThread.start ()
atexit.register (_pollThread.shutdown)
atexit.register (shutdown)

def selectevent (windowName, componentName, calendarEventName):
	"""Selects the row from the table of calendar events based on the calendar event name specified
	INPUT: selectevent ('<window name>', '<Calendar_view name>', '<calendar event name>')

	OUTPUT: Returns 1 on success and 0 on error."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectevent', _requestId, windowName, componentName, calendarEventName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectevent failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
			
def selecteventindex (windowName, componentName, eventNumber):
	"""Select an event from a calendar table using its index. Index for a calendar event starts from 1.
	INPUT: selecteventindex ('<window name>', '<component name>', <event number>)

	OUTPUT: Returns 1 on success and 0 on error."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecteventindex', _requestId, windowName, componentName, str (eventNumber))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecteventindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
			
def verifyeventexist (windowName, componentName):
	"""verifies whether any events are present in a calendar table
	INPUT: verifyeventexist ('<window name>', '<component name>')

	OUTPUT: Returns 1 on success and 0 on no events."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyeventexist', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def check (windowName, componentName):
	"""Check (tick) the check box state. 
	INPUT: check ('<window name>', '<component name>') 

	OUTPUT: Returns 1 if state is checked, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('check', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('check failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
			
def uncheck (windowName, componentName):
	"""Uncheck (un-tick) the check state.
	INPUT: uncheck ('<window name>', '<component name>')

	OUTPUT: Returns 1 if state is unchecked, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('uncheck', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('uncheck failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def click (windowName, componentName):
	"""click on radio button / check box / push button/ combo box/ radio menu item/ toggle button/ radio button.
	INPUT: click ('<window name>', '<component name>')

	OUTPUT: Clicks the component_name."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('click', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('click failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def menucheck (windowName, componentName):
	"""menucheck (tick) the menu item check state. 
	INPUT: menucheck ('<window name>', '<component name>') 

	OUTPUT: Returns 1 if state is checked, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('menucheck', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('menucheck failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def menuuncheck (windowName, componentName):
	"""menuuncheck (un-tick) the check state.
	INPUT: menuuncheck ('<window name>', '<component name>')

	OUTPUT: Returns 1 if state is unchecked, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('menuuncheck', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('menuuncheck failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifycheck (windowName, componentName):
	"""Checks the state of check box.
	INPUT: verifycheck ('<window name>', '<component name>')

	OUTPUT: If check box state is checked, then returns 1, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifycheck', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyuncheck (windowName, componentName):
	"""Checks the state of check box.
	INPUT: verifyuncheck ('<window name>', '<component name>')

	OUTPUT: If check box state is un-checked, then returns 1, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyuncheck', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifymenucheck (windowName, menuItem):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifymenucheck', _requestId, windowName, menuItem)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifymenuuncheck (windowName, menuItem):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifymenuuncheck', _requestId, windowName, menuItem)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifydropdown (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifydropdown', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

# CheckMenuItem Functions

def selectmenuitem (windowName, menuHierarchy):
	"""Selects the menu item specified.
	INPUT: selectmenuitem ('<window name>', '<menu hierarchy>')

	OUTPUT: TODO."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectmenuitem', _requestId, windowName, menuHierarchy)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectmenuitem failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

#  Combobox Functions

def hidelist (windowName, componentName):
	""" Hides combo box drop down list in the current dialog. 
	Suppose in previous operation one testcase has clicked on combo box, 
	its drop down list will be displayed. If further no any operation has 
	been done on that combo box then to close that drop down list 'HideList' 
	action is required.
	INPUT: hidelist ('windowName', 'componentName') 

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('hidelist', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('hidelist failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectindex (windowName, componentName, index):
	""" SelectIndex action will select an item from combo box where value of index is pointing to its position in list/menu.
	INPUT: selectindex ('windowName', 'componentName', <index>)

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectindex', _requestId, windowName, componentName, str (index))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def settextvalue (windowName, componentName, text):
	"""puts the text into the component given by the component name
	INPUT: settextvalue ('<window name>', '<component name>', '<text>')

	OUTPUT: returns 1 on success and 0 otherwise"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('settextvalue', _requestId, windowName, componentName, text)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('settextvalue failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def gettextvalue (windowName, componentName, startPosition = None, endPosition = None):
	"""
	INPUT: gettextvalue ('<window name>', '<component name>', [start position], [end position])

	OUTPUT: returns 1 on success and 0 otherwise"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('gettextvalue', _requestId, windowName, componentName, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('gettextvalue failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def activatetext (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('activatetext', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('activatetext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def appendtext (windowName, componentName, text):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('appendtext', _requestId, windowName, componentName, text)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('appendtext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getcursorposition (windowName, componentName):
	"""
	INPUT: windowName --> Name in the title bar
	       componentName --> Name of the object (TextBox)

	OUTPUT: postion of the text cursor in the object (integer)"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getcursorposition', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getcursorposition failed')
		return long (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setcursorposition (windowName, componentName, position):
	"""
	INPUT: windowName --> Name in the title bar
	       componentName --> Name of the object (TextBox)
	       position --> Position in the TextBox where cursor is to be moved to

	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setcursorposition', _requestId, windowName, componentName, str (position))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setcursorposition failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
	
def capturetofile (windowName, componentName, fileName = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('capturetofile', _requestId, windowName, componentName, fileName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('capturetofile failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def showlist (windowName, componentName):
	""" Displays combo box drop down list in the current dialog.
	INPUT: showlist ('windowName', 'componentName')

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('showlist', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('showlist failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verfyhidelist (windowName, componentName):
	""" Verifies if combo box drop down list in the current dialog is not visible.
	INPUT: verfyhidelist ('windowName', 'componentName')

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyhidelist', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyselect (windowName, componentName, selectArgument):
	""" VerfySelect action will verify if combo box is set to value given in argument.
	INPUT: verifyselect ('windowName', 'componentName', '<argument>')

	OUTPUT: VerifySelect function will try to find if text box associated with combo box
	is set to value specified in the argument."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyselect', _requestId, windowName, componentName, selectArgument)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifysettext (windowName, componentName, text):
	""" checks if the text is inserted into the component given by the component name
	INPUT: verifysettext ('<window name>', '<component name>', '<text>')

	OUTPUT: returns 1 if the text is inserted into the specified component else returns 0 """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifysettext', _requestId, windowName, componentName, text)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyshowlist (windowName, componentName):
	""" Verifies if combo box drop down list in the current dialog is visible.
	INPUT: verifyshowlist ('windowName', 'componentName')

	OUTPUT: Combo box will generally have a list as its child or a menu as its child. 
	So this function gets object handle of list or menu object, checks if list or 
	menu items of combo box is visible, if yes then return zero else minus one."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyshowlist', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def comboselect (windowName, componentName, menuItemName):
	"""Select a menu item or list item in a combo box
	INPUT: comboselect ('<window name>', '<component name>', '<menu item name>')

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('comboselect', _requestId, windowName, componentName, menuItemName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('comboselect failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def comboselectindex (windowName, componentName, index):
	"""
	INPUT: comboselectindex ('<window name>', '<component name>', <index>)

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('comboselectindex', _requestId, windowName, componentName, str (index))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('comboselectindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def comparetextproperty (windowName, componentName, textProperty, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('comparetextproperty', _requestId, windowName, componentName, textProperty, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def containstextproperty (windowName, componentName, textProperty, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('containstextproperty', _requestId, windowName, componentName, textProperty, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def gettextproperty (windowName, componentName, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('gettextproperty', _requestId, windowName, componentName, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('gettextproperty failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def copytext (windowName, componentName, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('copytext', _requestId, windowName, componentName, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('copytext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def cuttext (windowName, componentName, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('cuttext', _requestId, windowName, componentName, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('cuttext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def inserttext (windowName, componentName, position, text):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('inserttext', _requestId, windowName, componentName, str (position), text)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('inserttext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def deletetext (windowName, componentName, startPosition = None, endPosition = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('deletetext', _requestId, windowName, componentName, startPosition, endPosition)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('deletetext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selecttextbyindexandregion (windowName, componentName,
				startPosition = None, endPosition = None, selectionNum = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if startPosition != None:
		startPosition = str (startPosition)
	if endPosition != None:
		endPosition = str (endPosition)
	if selectionNum != None:
		selectionNum = str (selectionNum)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecttextbyindexandregion', _requestId, windowName, componentName, startPosition, endPosition, selectionNum)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecttextbyindexandregion failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selecttextbyname (windowName, textName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecttextbyname', _requestId, windowName, textName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecttextbyname failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def pastetext (windowName, componentName, position = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	if position != None:
		position = str (position)
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('pastetext', _requestId, windowName, componentName, position)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('pastetext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def expandtablecell (windowName, componentName, position):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('expandtablecell', _requestId, windowName, str (position))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('expandtablecell failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getcharcount (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getcharcount', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getcharcount failed')
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def geteventcount (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('geteventcount', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('geteventcount failed')
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getlabel (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getlabel', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getlabel failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getlabelatindex (windowName, componentName, index):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getlabelatindex', _requestId, windowName, componentName, index)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getlabelatindex failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectlabelspanelbyname (windowName, labelName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectlabelspanelbyname', _requestId, windowName, labelName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectlabelspanelbyname failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getpanelchildcount (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getpanelchildcount', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getpanelchildcount failed')
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getslidervalue (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getslidervalue', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getslidervalue failed')
		return float (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getstatusbartext (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getstatusbartext', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getstatusbartext failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def gettreetablerowindex (windowName, componentName, rowText):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('gettreetablerowindex', _requestId, windowName, componentName, rowText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('gettreetablerowindex failed')
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def grabfocus (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('grabfocus', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('grabfocus failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def istextstateenabled (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('istextstateenabled', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def listsubmenus (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('listsubmenus', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('listsubmenus failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def rightclick (windowName, componentName, menuItem, data = None):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('rightclick', _requestId, windowName, componentName, menuItem, data)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('rightclick failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectcalendardate (windowName, componentName, day, month, year):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectcalendardate', _requestId, windowName, componentName, str (day), str (month), str (year))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectcalendardate failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectitem (windowName, componentName, item):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectitem', _requestId, windowName, componentName, item)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectitem failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selecttextitem (windowName, componentName, item):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecttextitem', _requestId, windowName, componentName, item)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecttextitem failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setcellvalue (windowName, componentName, row, column, item):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setcellvalue', _requestId, windowName,
					componentName, str (row), str (column), item)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setcellvalue failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def sortcolumn (windowName, componentName, item):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('sortcolumn', _requestId, windowName, componentName, item)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('sortcolumn failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def sortcolumnindex (windowName, componentName, index):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('sortcolumnindex', _requestId, windowName, componentName, str (index))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('sortcolumnindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifybuttoncount (windowName, componentName, count):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifybuttoncount', _requestId, windowName, componentName, str (count))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

#  General Functions

def waittillguiexist (windowName):
	""" If the given window name exist, this function returns 1. If window doesnot exist, 
	then this function returns 0. Difference between guiexist and waitguiexist is, 
	waitguiexist waits for maximum 30 seconds. Still the window doesn't appear, 
	then 0 is returned. We can set the environment variable 'GUI_TIMEOUT' to change 
	the default waiting time.
	INPUT: waittillguiexist ('<window name>')

	OUTPUT: returns 1 on success and 0 on no existing window """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('waittillguiexist', _requestId, windowName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def waittillguinotexist (windowName):
	""" If the given window name does not exist, this function returns 1. If window exist, 
	then this function returns 0. Difference between guiexist and waitguinotexist is, 
	waitguinotexist waits for maximum 30 seconds. Still the window does not disappear, 
	then 0 is returned. We can set the environment variable 'GUI_TIMEOUT' to change the 
	default waiting time.
	INPUT: waittillguinotexist ('<window name>')

	OUTPUT: returns 1 on success and 0 on no existing window """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('waittillguinotexist', _requestId, windowName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def guiexist (windowName):
	""" If the given window name exist, this function returns 1. If window doesnot exist, 
	then this function returns 0.
	INPUT: guiexist ('<window name>') 

	OUTPUT: returns 1 on success and 0 on no existing window """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('guiexist', _requestId, windowName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def initappmap (appmapFileName):
	""" Application map will be loaded
	INPUT: initappmap ('<application map name>')

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('initappmap', _requestId, appmapFileName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('initappmap failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setappmap (appmapFileName):
	""" A new application map will be loaded. Existing appmap will be unloaded.
	INPUT: setappmap ('<new application map name>')

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setappmap', _requestId, appmapFileName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setappmap failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setcontext (oldContext, newContext):
	""" Set context of window
	INPUT: setcontext ('<old window title>', '<new window title>')

	OUTPUT: TODO"""

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setcontext', _requestId, oldContext, newContext)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setcontext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def releasecontext (oldContext, newContext):
	""" Release context of window
	INPUT: releasecontext (oldContext, newContext)

	OUTPUT: TODO"""

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('releasecontext', _requestId, oldContext, newContext)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('releasecontext failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def onwindowcreate (windowTitle, callbackFuncName):
	""" On new window creation event, notify
	INPUT: onwindowcreate ('<window title to watch>', <callback function to be called on window create event>)

	OUTPUT: TODO"""

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('onwindowcreate', _requestId, windowTitle)
		callback_functions[windowTitle] = callbackFuncName
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('onwindowcreate failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def reinitldtp ():
	""" Reinitialize LDTP, which will close existing at-spi connection
	and reestablish it.
	INPUT: reinitldtp ()

	OUTPUT: TODO"""

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('reinitldtp', _requestId)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('reinitldtp failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def remap (applicationName, windowName):
	""" We can handle dynamically created widgets (meaning widgets created at run time) using 
	this remap function. Calling remap will generate appmap for the given dialog at run 
	time and update the hash table. Then we can access the new widgets. But please make 
	sure to call undoremap() once the required functions are performed so that the hash 
	table will be reverted back to its original state. The reason for having undoremap() 
	is that subsequent calls to remap() might corrupt the hash table containg the appmap 
	entries.
	
	Please not that the <application-name> should be same as the one given as the commmand-line 
	argument for appmap generation. 
	INPUT: remap ('<application-name>', '<window name>')

	OUTPUT: It uses the same logic that appmap module uses to generate appmap."""

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('remap', _requestId, windowName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('remap failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

#  Log Function

def startlog (logFileName, fileOverWrite = 1):
	""" Start logging on the specified file. second arugment is optional and 1 is default value
	0 - Append log to an existing file
	1 - Write log to a new file. If file already exist, 
	then erase existing file content and start log

	INPUT: startlog ('<log file name>', [0 or 1])

	OUTPUT: Log file will be created if log file is not present in any case. If second argument is 1, 
	then existing file content will be erased. If second argument is 0, then new logs will be 
	append to existing log.

	It returns 1 on Success and 0 on error  """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('startlog', _requestId, logFileName, str (fileOverWrite))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			pass
		return 1
	except LdtpExecutionError, msg:
		pass

def log (message, priority = ''):
	""" Logs the message in the log.xml with the tag which can be viewed after the execution of scripts
	INPUT: log('<Message to be logged>','<tag>')

	OUTPUT: The required message will be logged into the log.xml on execution of scripts
	It returns 1 on Success and 0 on error  """

	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('log', _requestId, message, priority)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			pass
		return 1
	except ConnectionLost:
		pass
	except LdtpExecutionError:
		pass

def stoplog ():
	""" Stop logging.
	INPUT: stoplog()

	OUTPUT: If a log file has been previously opened for logging, that file pointer will be closed. 
	So that the new logging will not be appened to the log file. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('stoplog', _requestId)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			pass
		return 1
	except ConnectionLost:
		pass
	except LdtpExecutionError:
		pass

#  MenuItem Functions

# Table Functions

def selectrow (windowName, tableName, rowText):
	"""Selects the row with the text in the table
	INPUT: selectevent ('<window name>', '<table name>', '<row text to be selected>')

	OUTPUT: Returns 1 on success and 0 on error."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectrow', _requestId, windowName, tableName, rowText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectrow failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
	
def selectrowindex (windowName, tableName, rowIndex):
	"""Selects the row with the index in the table
	INPUT: selectevent ('<window name>', '<table name>', <row index>)

	OUTPUT: Returns 1 on success and 0 on error."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectrowindex', _requestId, windowName, tableName, str(rowIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectrowindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

#  MenuItem Functions

def doesmenuitemexist (windowName, menuHierarchy):
	""" checks if the specified menuitem specified in the menu hierarchy is present or not
	INPUT: doesmenuitemexist ('<window name>', '<menu hierarchy>')

	OUTPUT: Returns 1 if the menuitem is present and 0 otherwise """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('doesmenuitemexist', _requestId, windowName, menuHierarchy)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

#  Panel Functions

def selectpanel (windowName, componentName, panelNumber):
	""" Select a panel using the panel number in a list of panels
	INPUT: selectpanel ('<window name>', '<component name>', <panel number>)

	OUTPUT: Returns 1 on success and 0 on error """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectpanel', _requestId, windowName, componentName, str(panelNumber))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectpanel failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectpanelname (windowName, componentName, panelName):
	""" Select a panel using the panel name in a list of panels
	INPUT: selectpanelname ('<window name>', '<component name>', <panel name>)

	OUTPUT: Returns 1 on success and 0 on error """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectpanelname', _requestId, windowName, componentName, panelName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectpanelname failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

#  PushButton Functions

def verifypushbutton (windowName, componentName):
	""" Verify whether the given object is push button or not.
	INPUT: verifypushbutton ('<window name>', '<component name>')

	OUTPUT: Returns 1 if object is push button, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifypushbutton', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def stateenabled (windowName, componentName):
	""" Checks the radio button object state enabled or not
	INPUT: stateenabled ('<window name>', '<component name>')

	OUTPUT: Returns 1 if state is enabled, else 0."""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('stateenabled', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

#  ScrollBar Functions

def onedown (windowName, componentName, iterations):
	""" Move the scroll bar down 'n' times, where 'n' is the number of iterations 
	specified in  the argument field.
	INPUT: onedown ('<window name>', '<scroll component name>', '<number of iterations>')

	OUTPUT: Scrolls down if value does not exceed the maximum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('onedown', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('onedown failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def oneleft (windowName, componentName, iterations):
	""" Move the (horizontal) scroll bar left 'n' times, where 'n' is the number of iterations 			specified in the argument field.
	INPUT: oneleft ('<window name>', '<scroll component name>', '<number of iterations>')

	OUTPUT: Scrolls left if value does not exceed the maximum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('oneleft', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('oneleft failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def oneright (windowName, componentName, iterations):
	""" Move the (horizontal) scroll bar right 'n' times, where 'n' is the number of iterations 			specified in the argument field.
	INPUT: oneright ('<window name>', '<scroll component name>', '<number of iterations>')

	OUTPUT: Scrolls right if value does not exceed the maximum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('oneright', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('oneright failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def oneup (windowName, componentName, iterations):
	""" Move the scroll bar up 'n' times, where 'n' is the number of iterations 
	specified in  the argument field.
	INPUT: oneup ('<window name>', '<scroll component name>', '<number of iterations>')

	OUTPUT: Scrolls up if value does not exceed the maximum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('oneup', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('oneup failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
    
def scrolldown (windowName, componentName):
	""" Move the (vertical) scroll bar to the bottom.
	INPUT: scrolldown ('<window name>', '<scroll component name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('scrolldown', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('scrolldown failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def scrollleft (windowName, componentName):
	""" Move the (horizontal) scroll bar to the left.
	INPUT: scrollleft ('<window name>', '<scroll component name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('scrollleft', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('scrollleft failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def scrollright (windowName, componentName):
	""" Move the (horizontal) scroll bar to the right.
	INPUT: scrollright ('<window name>', '<scroll component name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('scrollright', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('scrollright failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
	
def scrollup (windowName, componentName):
	""" Move the (vertical) scroll bar to the top.
	INPUT: scrollup ('<window name>', '<scroll component name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('scrollup', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('scrollup failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifyscrollbar (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyscrollbar', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyscrollbarvertical (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyscrollbarvertical', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyscrollbarhorizontal (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyscrollbarhorizontal', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyslider (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyslider', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyslidervertical (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyslidervertical', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifysliderhorizontal (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifysliderhorizontal', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifystatusbar (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifystatusbar', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifystatusbarvisible (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifystatusbarvisible', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifytoggled (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifytoggled', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifyvisiblebuttoncount (windowName, componentName, count):
	"""
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyvisiblebuttoncount', _requestId, windowName, componentName, str (count))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

#  Slider Functions

def decrease (windowName, componentName, iterations):
	""" Decrease the value of the slider 'n' times, where 'n' is the number of 
    	iterations specified in the argument field.
	INPUT: decrease ('<window name>', '<slider name>', '<number of iterations>')

	OUTPUT: Decreases the value if it does not fall below the minimum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('decrease', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('decrease failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def increase (windowName, componentName, iterations):
	""" Increase the value of the slider 'n' times, where 'n' is the number of 
    	iterations specified in the argument field.
	INPUT: increase ('<window name>', '<slider name>', '<number of iterations>')

	OUTPUT: Increases the value if it does not fall below the minimum limit, else fails. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('increase', _requestId, windowName, componentName, str (iterations))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('increase failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setmax (windowName, componentName):
	""" Set the slider to the maximum value.
	INPUT: setmax ('<window name>', '<slider name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setmax', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setmax failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def setmin (windowName, componentName):
	""" Set the slider to the minimum value.
	INPUT: setmin ('<window name>', '<slider name>')

	OUTPUT: Returns 1 if action is performed, else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setmin', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setmin failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

#  SpinButton Functions

def setvalue (windowName, componentName, value):
	""" Sets the value of the spin button.
	INPUT: setvalue ('<window name>', '<spinbutton name>', '<value>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setvalue', _requestId, windowName, componentName, str (value))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('setvalue failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getvalue (windowName, componentName):
	""" Gets the value in the spin button.
	INPUT: getvalue ('<window name>', '<spinbutton name>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getvalue', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getvalue failed')
		return float (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifysetvalue (windowName, componentName, value) :
	""" Verifies the value set in spin button.
	INPUT: verifysetvalue ('<window name>', '<spinbutton name>', '<value>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('setvalue', _requestId, windowName, componentName, str (value))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

#  TabControl Functions

def selecttab (windowName, componentName, tabName):
	""" Select the given tab name in the tab list
	INPUT: selecttab ('<window name>', '<tab list name>', '<tab name>')

	OUTPUT: Returns 1 if the tab is selected, otherwise exception will be thrown """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecttab', _requestId, windowName, componentName, tabName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecttab failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selecttabindex (windowName, componentName, tabIndex):
	""" Select a particular tab in the list of tabs
	INPUT: selecttabindex ('<windwo name>', '<tab list name>', <index of the tab>)

	OUTPUT: Returns 1 if the tab is selected, otherwise exception will be thrown """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selecttabindex', _requestId, windowName, componentName, str (tabIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selecttabindex failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def gettabcount (windowName, componentName):
	""" Gets the number of tabs in an object
	INPUT: gettabcount ('<window name>', '<object name>')
	       e.g. getttabcount ('terminal','ptl0')

	OUTPUT: Returns the number of tabs in the object """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('gettabcount', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('gettabcount failed')
		return long(_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def checkrow (windowName, componentName, rowIndex, columnIndex = 0):
	""" checks the row with the given index value in table. This can take an 
	optional column index and perform the action on that particular column. 
	If the column index is not given, 0 is taken as the default value.
	Index value starts from 0.
	INPUT: checkrow ('<window name>', '<table name>', <row index>, <col index>)

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('checkrow', _requestId, windowName, componentName, str (rowIndex), str (columnIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('checkrow failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifycheckrow (windowName, componentName, rowIndex, columnIndex = 0):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifycheckrow', _requestId, windowName, componentName, str (rowIndex), str (columnIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def doesrowexist (windowName, componentName, rowText):
	""" Checks whether the table contains any row with any of its cell containing 
	the given string as its value.Please note that it checks for an exact match.
	INPUT: doesrowexist ('<window name>', '<table name>', '<string to be matched>')

	OUTPUT: Returns 1 if there are rows with the given string in any of its cell else 0. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('doesrowexist', _requestId, windowName, componentName, rowText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0
    
def doubleclick (windowName, componentName):
	"""
	INPUT: doubleclick ('<window name>', '<table name>')

	OUTPUT: Returns 1 on success and exception on error"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('doubleclick', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('doubleclick failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def doubleclickrow (windowName, componentName, rowText):
	""" Double clicks the row in table whose first column's (0th column) value 
	is same as the contents of the third argument in the function call.
	INPUT: doubleclickrow ('<window name>', '<table name>', '<value of row in first column>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('doubleclickrow', _requestId, windowName, componentName, rowText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('doubleclickrow failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getcellvalue (windowName, componentName, row, column = 0):
	""" returns the text in a cell at given row and column of a tree table
	INPUT: getcellvalue ('<window name>', '<component name>', '<row>', '<column>')

	OUTPUT: returns the string """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getcellvalue', _requestId, windowName, componentName, str (row), str (column))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getcellvalue failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getrowcount (windowName, componentName):
	""" returns the number of rows in the table
	INPUT: getrowcount ('<window name>', '<table name>' )

	OUTPUT: Returns the number of rows present in the table mentioned """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getrowcount', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getrowcount failed')
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))
    
def gettablerowindex (windowName, componentName, rowText):
	""" Returns the id of the row containing the given <cellvalue>
	INPUT: gettablerowindex ('<window name>','<tablename>','<cellvalue>')

	OUTPUT: Return id of the row containing the given cell value, if it is found else return -1 """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('gettablerowindex', _requestId, windowName, componentName, rowText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return -1L
		return int (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectlastrow (windowName, componentName):
	""" Selects the last row of a table.
	INPUT: selectlastrow ('<window name>', '<table name>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectlastrow', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectlastrow failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def selectrowpartialmatch (windowName, componentName, rowPartialText):
	""" selects the row having cell that contains the given text.
	INPUT: selectrowpartialmatch ('<window name>', '<tree table name>', '<texttobesearchedfor>')

	OUTPUT: returns 1 on success and throws an exception on error """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('selectrowpartialmatch', _requestId, windowName, componentName, rowPartialText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('selectrowpartialmatch failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifypartialmatch (windowName, componentName, rowPartialText):
	""" 
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifypartialmatch', _requestId, windowName, componentName, rowPartialText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifypartialtablecell (windowName, componentName, rowIndex, columnIndex, rowPartialText):
	""" 
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifypartialtablecell', _requestId, windowName, componentName,
					str (rowIndex), str (columnIndex), rowPartialText)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def uncheckrow (windowName, componentName, rowIndex, columnIndex = 0):
	""" Unchecks the row with the given index value in table. This can take an 
	optional column index and perform the action on that particular column. 
	If the column index is not given, 0 is taken as the default value. Index 
	value starts from 0.
	INPUT: uncheckrow ('<window name>', '<table name>', <row index>, [<col index>])

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('uncheckrow', _requestId, windowName, componentName, str (rowIndex), str (columnIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('uncheckrow failed')
		return 1
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def verifyuncheckrow (windowName, componentName, rowIndex, columnIndex = 0):
	""" 
	INPUT: TODO

	OUTPUT: TODO """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifyuncheckrow', _requestId, windowName, componentName, str (rowIndex), str (columnIndex))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifytablecell (windowName, componentName, rowIndex, columnIndex, textToCompare):
	""" Verifies the tablecell value with the String Passed ie., fifth argument
	INPUT: verifytablecell ('<window name>', '<table name>', '<row no>','<column no>','<string to be compared>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifytablecell', _requestId, windowName, componentName,
					str (rowIndex), str (columnIndex), textToCompare)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0

def verifypartialcellvalue (windowName, componentName, rowIndex, columnIndex, partialTextToCompare):
	""" Verifies the tablecell value with the sub String Passed ie., fifth argument
	INPUT: verifypartialtablecell ('<window name>', '<table name>', '<row no>','<column no>','< sub string to be compared>')

	OUTPUT: Returns 1 on success and 0 on error. """
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('verifypartialcellvalue', _requestId, windowName, componentName,
					str (rowIndex), str (columnIndex), partialTextToCompare)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0
    
def getwindowlist ():
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getwindowlist', _requestId)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getwindowlist failed')
		return parseobjectlist (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getobjectlist (windowName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getobjectlist', _requestId, windowName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getobjectlist failed')
		return parseobjectlist (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getobjectinfo (windowName, componentName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getobjectinfo', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getobjectinfo failed')
		return parseobjectlist (_responseData[1])
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def getobjectproperty (windowName, componentName, propertyName):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('getobjectproperty', _requestId, windowName, componentName, propertyName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('getobjectproperty failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def bindtext (packageName, localeDir, mode = 'mo'):
	"""
	INPUT: TODO

	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('bindtext', _requestId, packageName, localeDir, mode)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('bindtext failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def mouseleftclick (windowName, componentName):
	"""
	INPUT: windowName --> Window containing object
               componentName --> object's name

	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('mouseleftclick', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('mouseleftclick failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def mouserightclick (windowName, componentName):
	"""
	INPUT: windowName --> Window containing object
               componentName --> object's name

	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('mouserightclick', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('mouserightclick failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def mousemove (windowName, componentName):
	"""
	INPUT: windowName --> Window containing object
               componentName --> object's name

	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('mousemove', _requestId, windowName, componentName)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('mousemove failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def enterstring (windowName, componentName, data):
	"""
	INPUT: windowName --> Window containing object
               componentName --> object's name
	       data --> string to be simulated as entered from keyboard
	                Special non printing characters are enter within '<' and '>'
			e.g. <capslock>,<ctrl>, etc while other characters are entered
			in a straight forward fashion. e.g. '<ctrl>Nwww.google.com'    
	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('enterstring', _requestId, windowName, componentName, data)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			raise LdtpExecutionError ('enterstring failed')
		return _responseData[1]
	except LdtpExecutionError, msg:
		raise LdtpExecutionError (str (msg))

def stopscriptengine ():
	"""
	OUTPUT: None"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		_message = generatexml ('stopscriptengine', _requestId)
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
	except LdtpExecutionError, msg:
		pass

def hasstate (windowName, componentName, state):
	"""
	INPUT: TODO
	OUTPUT: TODO"""
	try:
		_requestId  = str (random.randint (0, sys.maxint))
		p = re.compile(r'\W+')
		_message = generatexml ('hasstate', _requestId, windowName, componentName, p.split (state))
		sendpacket (_message)
		_responseStatus, _responseData = getresponse (_requestId)
		if _responseStatus[0] != 0:
			return 0
		return 1
	except LdtpExecutionError, msg:
		return 0
